Package: OncoBayes2
Type: Package
Title: Bayesian Logistic Regression for Oncology Dose-Escalation Trials
Description: Bayesian logistic regression model with optional
  EXchangeability-NonEXchangeability parameter modelling for flexible
  borrowing from historical or concurrent data-sources. The safety model
  can guide dose-escalation decisions for adaptive oncology Phase I
  dose-escalation trials which involve an arbitrary number of
  drugs. Please refer to Neuenschwander et al. (2008)
  <doi:10.1002/sim.3230> and Neuenschwander et al. (2016)
  <doi:10.1080/19466315.2016.1174149> for details on the methodology.
Version: 0.9-4
Date: 2025-12-17
Authors@R: c(person("Novartis", "Pharma AG", role = "cph")
            ,person("Sebastian", "Weber", email="sebastian.weber@novartis.com", role=c("aut", "cre"))
            ,person("Lukas A.", "Widmer", email="lukas_andreas.widmer@novartis.com", role="aut")
            ,person("Andrew", "Bean", email="andrew.bean@novartis.com", role="aut")
            ,person("Trustees of", "Columbia University", role="cph", comment="R/stanmodels.R, configure, configure.win")
	    )
Depends: R (>= 3.5.0), RBesT (>= 1.8-0)
Imports: methods, Rcpp (>= 0.12.0), RcppParallel (>= 5.0.1), rstan (>=
        2.32.0), rstantools (>= 2.4.0), posterior (>= 1.5.0),
        assertthat (>= 0.2.1), checkmate, Formula, bayesplot (>=
        1.4.0), ggplot2 (>= 3.0.0), dplyr (>= 1.1.0), tibble, tidyr (>=
        1.0.0), abind, scales, rlang (>= 0.3.0), tidyselect, utils,
        stats, matrixStats, brms, lifecycle, grDevices
LinkingTo: BH (>= 1.75.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.32.0), StanHeaders (>=
        2.32.0)
License: GPL (>= 3)
LazyData: true
NeedsCompilation: yes
UseLTO: true
URL: https://opensource.nibr.com/OncoBayes2/
BugReports: https://github.com/Novartis/OncoBayes2/issues
Suggests: rmarkdown, knitr, testthat (>= 2.0.0), withr, mvtnorm,
        vdiffr, cmdstanr (>= 0.4.0), ragg
Additional_repositories: https://stan-dev.r-universe.dev
VignetteBuilder: knitr
Biarch: true
SystemRequirements: GNU make, pandoc (>= 1.12.3), pngquant, C++17
Encoding: UTF-8
RoxygenNote: 7.3.3
Packaged: 2025-12-18 08:49:05 UTC; weberse2
Author: Novartis Pharma AG [cph],
  Sebastian Weber [aut, cre],
  Lukas A. Widmer [aut],
  Andrew Bean [aut],
  Trustees of Columbia University [cph] (R/stanmodels.R, configure,
    configure.win)
Maintainer: Sebastian Weber <sebastian.weber@novartis.com>
Repository: CRAN
Date/Publication: 2025-12-18 12:10:10 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 20:15:04 UTC; unix
Archs: OncoBayes2.so.dSYM
