# Generated by roxygen2: do not edit by hand

S3method(as_draws,blrm_trial)
S3method(as_draws,blrmfit)
S3method(as_draws_array,blrm_trial)
S3method(as_draws_array,blrmfit)
S3method(as_draws_df,blrm_trial)
S3method(as_draws_df,blrmfit)
S3method(as_draws_list,blrm_trial)
S3method(as_draws_list,blrmfit)
S3method(as_draws_matrix,blrm_trial)
S3method(as_draws_matrix,blrmfit)
S3method(as_draws_rvars,blrm_trial)
S3method(as_draws_rvars,blrmfit)
S3method(critical_quantile,blrm_trial)
S3method(critical_quantile,blrmfit)
S3method(critical_quantile,default)
S3method(log_posterior,blrmfit)
S3method(model.matrix,blrmfit)
S3method(neff_ratio,blrmfit)
S3method(nsamples,blrm_trial)
S3method(nsamples,blrmfit)
S3method(nuts_params,blrmfit)
S3method(plot_toxicity_curve,blrm_trial)
S3method(plot_toxicity_curve,blrmfit)
S3method(plot_toxicity_curve,default)
S3method(plot_toxicity_intervals,blrm_trial)
S3method(plot_toxicity_intervals,blrmfit)
S3method(plot_toxicity_intervals,default)
S3method(plot_toxicity_intervals_stacked,blrm_trial)
S3method(plot_toxicity_intervals_stacked,blrmfit)
S3method(plot_toxicity_intervals_stacked,default)
S3method(posterior_interval,blrm_trial)
S3method(posterior_interval,blrmfit)
S3method(posterior_linpred,blrm_trial)
S3method(posterior_linpred,blrmfit)
S3method(posterior_predict,blrm_trial)
S3method(posterior_predict,blrmfit)
S3method(predictive_interval,blrm_trial)
S3method(predictive_interval,blrmfit)
S3method(print,blrm_trial)
S3method(print,blrmfit)
S3method(print,prior_summary.blrm_trial)
S3method(print,prior_summary.blrmfit)
S3method(prior_summary,blrm_trial)
S3method(prior_summary,blrmfit)
S3method(rhat,blrmfit)
S3method(summary,blrm_trial)
S3method(summary,blrmfit)
S3method(update,blrm_trial)
S3method(update,blrmfit)
export(as_draws)
export(as_draws_array)
export(as_draws_df)
export(as_draws_list)
export(as_draws_matrix)
export(as_draws_rvars)
export(bind_rows_0)
export(blrm_exnex)
export(blrm_formula_linear)
export(blrm_formula_saturating)
export(blrm_trial)
export(critical_quantile)
export(example_model)
export(inv_logit)
export(log_posterior)
export(logit)
export(neff_ratio)
export(nsamples)
export(nuts_params)
export(plot_toxicity_curve)
export(plot_toxicity_intervals)
export(plot_toxicity_intervals_stacked)
export(posterior_interval)
export(posterior_linpred)
export(posterior_predict)
export(predictive_interval)
export(prior_summary)
export(rhat)
import(Formula)
import(RBesT)
import(Rcpp)
import(abind)
import(assertthat)
import(checkmate)
import(dplyr)
import(ggplot2)
import(methods)
import(rstantools)
import(tidyr)
importFrom(RcppParallel,CxxFlags)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(bayesplot,log_posterior)
importFrom(bayesplot,neff_ratio)
importFrom(bayesplot,nuts_params)
importFrom(bayesplot,rhat)
importFrom(lifecycle,deprecate_stop)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(matrixStats,logSumExp)
importFrom(posterior,"%**%")
importFrom(posterior,"variables<-")
importFrom(posterior,as_draws)
importFrom(posterior,as_draws_array)
importFrom(posterior,as_draws_df)
importFrom(posterior,as_draws_list)
importFrom(posterior,as_draws_matrix)
importFrom(posterior,as_draws_rvars)
importFrom(posterior,as_rvar)
importFrom(posterior,bind_draws)
importFrom(posterior,default_convergence_measures)
importFrom(posterior,draws_of)
importFrom(posterior,ess_mean)
importFrom(posterior,ess_quantile)
importFrom(posterior,mcse_mean)
importFrom(posterior,mcse_quantile)
importFrom(posterior,ndraws)
importFrom(posterior,nvariables)
importFrom(posterior,resample_draws)
importFrom(posterior,rvar)
importFrom(posterior,rvar_ifelse)
importFrom(posterior,rvar_rng)
importFrom(posterior,subset_draws)
importFrom(posterior,summarise_draws)
importFrom(posterior,variables)
importFrom(rlang,.data)
importFrom(rlang,env_bury)
importFrom(rstan,get_adaptation_info)
importFrom(rstan,get_divergent_iterations)
importFrom(rstan,get_sampler_params)
importFrom(rstan,get_stancode)
importFrom(rstan,sampling)
importFrom(rstan,stanc_builder)
importFrom(rstan,summary)
importFrom(scales,extended_breaks)
importFrom(scales,number_format)
importFrom(stats,.getXlevels)
importFrom(stats,as.formula)
importFrom(stats,dbinom)
importFrom(stats,delete.response)
importFrom(stats,ftable)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.matrix.default)
importFrom(stats,model.response)
importFrom(stats,na.fail)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,update.default)
importFrom(tibble,as_tibble)
importFrom(tidyselect,vars_select)
importFrom(tidyselect,vars_select_helpers)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,modifyList)
useDynLib(OncoBayes2, .registration = TRUE)
